#include "slidecamera.h"

//////////////////////////////////////////////////////////////////////
// CSLIDECamera Class
//

CSLIDECamera::CSLIDECamera()
{
  m_dyniProjection.SetValue(slidepjtSLF_PARALLEL);
  m_vFrustumMin.Null();
  m_vFrustumMax.Null();
}

CSLIDECamera::~CSLIDECamera()
{
}

VOID CSLIDECamera::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
			CHAR *pcID)
{
  CSLIDEPrimitive::Init(pcFile, uiLine, uiColumn, pcID);

  m_dyniProjection.SetValue(slidepjtSLF_PARALLEL);
  m_vFrustumMin.Set(-1.0, -1.0, -100.0);
  m_vFrustumMax.Set(1.0, 1.0, (FLOAT)-0.01);
}

VOID CSLIDECamera::Uninit()
{
  CSLIDEPrimitive::Uninit();
}

BOOL CSLIDECamera::Initialized()
{
  return CSLIDEPrimitive::Initialized();
}

BOOL CSLIDECamera::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  return CSLIDEPrimitive::CheckSemantics(pCSLIDEWorld);
}

BOOL CSLIDECamera::Valid()
{
  if ( !CSLIDEPrimitive::Valid() )
    {
      return FALSE;
    }

  return ( 0 <= m_dyniProjection.GetValue() &&
	   m_dyniProjection.GetValue() < slidepjtSize);
}

CSLIDECamera *CSLIDECamera::FindCamera(CListIterInt *piterIDs, 
				       CListInt *plistCSLIDEInstance)
{
  CHAR *pcID;
  CSLIDECamera *pCSLIDECamera;

  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  pCSLIDECamera = this;

  pcID = (CHAR*)piterIDs->PeekNext();
  if ( pcID == NULL )
    {
      pcID = (CHAR*)piterIDs->PeekFirst();
      if ( pcID != GetID() )
	{
	  HandleError("Bad %s path: %s id \"%s\" not specified", 
		      gcppcSLIDEReservedWords[sliderwtCamera],
		      gcppcSLIDEReservedWords[sliderwtCamera], GetID());
	  pCSLIDECamera = NULL;
	}
    }
  else if ( pcID != GetID() )
    {
      HandleError("Bad %s path: path id \"%s\" != %s id \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtCamera], pcID,
		  gcppcSLIDEReservedWords[sliderwtCamera], GetID());
      pCSLIDECamera = NULL;
    }

  return pCSLIDECamera;
}

CSLIDELight *CSLIDECamera::FindLight(CListIterInt *piterIDs, 
				     CListInt *plistCSLIDEInstance)
{
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  HandleError("Bad %s path: %s \"%s\" encountered", 
	      gcppcSLIDEReservedWords[sliderwtLight],
	      gcppcSLIDEReservedWords[sliderwtCamera], GetID());

  return NULL;
}

VOID CSLIDECamera::CheckDynamic()
{
  SetDynamic(m_dyniProjection.GetDynamic() ||
	     m_vFrustumMin.GetDynamic() ||
	     m_vFrustumMax.GetDynamic());
}

VOID CSLIDECamera::UpdateState()
{
  m_dyniProjection.Update();
  if ( !( 0 <= m_dyniProjection.GetValue() &&
	  m_dyniProjection.GetValue() < slidepjtSize ) )
    {
      HandleError("Illegal Projection flag value:  %ld", 
		  m_dyniProjection.GetValue());
      m_dyniProjection.SetValue(slidepjtSLF_PARALLEL);
    }

  m_vFrustumMin.Update();
  m_vFrustumMax.Update();
}

VOID CSLIDECamera::Render(const Matrix &/*mGvp_proj*/, 
			  const Matrix &/*mGproj_vrc*/, 
			  const Matrix &/*mGvrc_proj*/, 
			  const Matrix &/*mGvrc_obj*/, 
			  const Matrix &/*mGobj_vrc*/, 
			  CBound * /*pCBoundClip*/, 
			  CListInt * /*plistLights*/,
			  CSLIDESurface * /*pCSLIDESurface*/,
			  SLIDEShadingType /*eShading*/, 
			  SLIDELODType /*eLOD*/)
{
}

VOID CSLIDECamera::WriteSLIDE(FILE *fpOutputFile)
{
  ASSERT( fpOutputFile != NULL );

  fprintf(fpOutputFile, "%s %s\n",
	  gcppcSLIDEReservedWords[sliderwtCamera], m_pcID);
  fprintf(fpOutputFile, "  %s %s\n",
	  gcppcSLIDEReservedWords[sliderwtProjection], 
	  gcppcSLIDEProjectionReservedWords[m_dyniProjection.GetValue()]
	  ); 
  fprintf(fpOutputFile, "  %s ( %.3f %.3f %.3f ) ( %.3f %.3f %.3f )\n", 
	  gcppcSLIDEReservedWords[sliderwtFrustum], 
	  m_vFrustumMin[X], m_vFrustumMin[Y], m_vFrustumMin[Z],
	  m_vFrustumMax[X], m_vFrustumMax[Y], m_vFrustumMax[Z]);
  fprintf(fpOutputFile, "%s\n",
	  gcppcSLIDEReservedWords[sliderwtEndCamera]);
}

//////////////////////////////////////////////////////////////////////
// CSLIDECameraPath Class
//

CSLIDECameraPath::CSLIDECameraPath()
{
}

CSLIDECameraPath::~CSLIDECameraPath()
{
}

CSLIDENode *CSLIDECameraPath::FindNode(CSLIDENode *pRoot, CListIterInt *piterIDs, 
				       CListInt *plistCSLIDEInstance)
{
  ASSERT( pRoot != NULL );
  ASSERT( piterIDs != NULL );
  ASSERT( plistCSLIDEInstance != NULL );

  return pRoot->FindCamera(piterIDs, plistCSLIDEInstance);
}

