#include "slideface.h"

#include "slideworld.h"
#include "slidepoint.h"
#include "slidesurface.h"

CListInt *CSLIDEFace::m_plistVertices = NULL;
UINT_32 CSLIDEFace::m_uiCount = 0;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEFace Class
//

CSLIDEFace::CSLIDEFace()
{
  m_pcSurfaceID = NULL;
  m_pCSLIDESurface = NULL;
  m_plistPointIDs = NULL;
}

CSLIDEFace::~CSLIDEFace()
{
}

VOID CSLIDEFace::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, CHAR *pcID, 
		      CListInt *plistPointIDs)
{
  CSLIDEPrimitive::Init(pcFile, uiLine, uiColumn, pcID);

  ASSERT( plistPointIDs != NULL );

  m_uiCount++;
  if ( m_plistVertices == NULL )
    {
      m_plistVertices = new CListInt;
      ASSERT( m_plistVertices != NULL );
      m_plistVertices->Init();
    }

  m_pcSurfaceID = gcppcSLIDEReservedWords[sliderwtSLF_INHERIT];
  m_pCSLIDESurface = NULL;

  m_plistPointIDs = plistPointIDs;
  m_listCSLIDEPoint.Init();
}

VOID CSLIDEFace::Uninit()
{
  CSLIDEPrimitive::Uninit();

  m_uiCount--;
  if ( m_uiCount == 0 )
    {
      if ( m_plistVertices != NULL )
	{
	  delete m_plistVertices;
	  m_plistVertices = NULL;
	}
    }

  if ( m_plistPointIDs != NULL )
    {
      delete m_plistPointIDs;
      m_plistPointIDs = NULL;
    }
}

BOOL CSLIDEFace::Initialized()
{
  if ( !CSLIDEPrimitive::Initialized() )
    {
      return FALSE;
    }

  return ( m_pcSurfaceID != NULL &&
	   m_plistPointIDs != NULL );
}

BOOL CSLIDEFace::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  BOOL bCorrect;
  CListIterInt iter;
  CHAR *pcID;
  CSLIDEPoint *pCSLIDEPoint;

  if ( !CSLIDEPrimitive::CheckSemantics(pCSLIDEWorld) )
    {
      return FALSE;
    }

  bCorrect = TRUE;

  // Check Surface Reference

  m_pCSLIDESurface = pCSLIDEWorld->PeekSurface(m_pcSurfaceID);
  if ( m_pCSLIDESurface == NULL )
    {
      HandleError("Undefined %s \"%s\"", 
		  gcppcSLIDEReservedWords[sliderwtSurface],
		  m_pcSurfaceID);
      bCorrect = FALSE;
    }

  // Check Point References

  iter.Init(m_plistPointIDs);

  for ( pcID = (CHAR*)(iter.PeekFirst());
	pcID != NULL;
	pcID = (CHAR*)(iter.PeekNext()) )
    {
      pCSLIDEPoint = pCSLIDEWorld->PeekPoint(pcID);
      if ( pCSLIDEPoint == NULL )
	{
	  HandleError("Undefined %s \"%s\"", gcppcSLIDEReservedWords[sliderwtPoint], 
		      pcID);
	  bCorrect = FALSE;
	}
      m_listCSLIDEPoint.InsertLast(pCSLIDEPoint, 0);
    }

  return bCorrect;
}

BOOL CSLIDEFace::Valid()
{
  if ( !CSLIDEPrimitive::Valid() )
    {
      return FALSE;
    }

  return (GetPointCount() >= 3);
}

CSLIDEPrimitiveIter *CSLIDEFace::AllocateIter()
{
  CSLIDEFaceIter *pCSLIDEFaceIter;

  pCSLIDEFaceIter = new CSLIDEFaceIter;
  ASSERT(pCSLIDEFaceIter != NULL );
  pCSLIDEFaceIter->Init(this);

  return pCSLIDEFaceIter;
}

VOID CSLIDEFace::FreeIter(CSLIDEPrimitiveIter *pCSLIDEFaceIter)
{
  delete pCSLIDEFaceIter;
}

////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: Fill these in
// 
VOID CSLIDEFace::InsertIntoBound(CBound *pCBound)
{
}

VOID CSLIDEFace::Preprocess()
{
}

VOID CSLIDEFace::CheckDynamic()
{
}

VOID CSLIDEFace::UpdateState()
{
}

VOID CSLIDEFace::Render(const Matrix &mGvp_proj, 
			const Matrix &mGproj_vrc, 
			const Matrix &/*mGvrc_proj*/, 
			const Matrix &mGvrc_obj, 
			const Matrix &/*mGobj_vrc*/, 
			CBound * /*pCBoundClip*/, 
			CListInt * /*plistCSLIDELightPath*/,
			CSLIDESurface *pCSLIDESurface,
			SLIDEShadingType /*eShading*/, SLIDELODType /*eLOD*/, 
			SLIDESolidType /*eSolid*/)
{
  Matrix mGvp_obj;

  // Hint
  mGvp_obj = mGvp_proj * mGproj_vrc * mGvrc_obj; 
}

VOID CSLIDEFace::ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface)
{
}
// 
// END  : ASSIGNMENT 3: Fill these in
////////////////////////////////////////////////////////////////////////

VOID CSLIDEFace::WriteSLIDE(FILE *fpOutputFile)
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;

  ASSERT( fpOutputFile != NULL );

  fprintf(fpOutputFile, "%s %s ( ", gcppcSLIDEReservedWords[sliderwtFace], 
	  m_pcID);

  iter.Init(&m_listCSLIDEPoint);

  for ( pCSLIDEPoint = (CSLIDEPoint*)iter.PeekFirst(); 
	pCSLIDEPoint != NULL;
	pCSLIDEPoint = (CSLIDEPoint*)iter.PeekNext() )
    {
      fprintf(fpOutputFile, "%s ", pCSLIDEPoint->GetID());      
    }

  fprintf(fpOutputFile, ")\n");

  if ( m_pcSurfaceID != gcppcSLIDEReservedWords[sliderwtSLF_INHERIT] )
    {
      fprintf(fpOutputFile, "  %s %s\n", 
	      gcppcSLIDEReservedWords[sliderwtSurface], 
	      m_pcSurfaceID);
    } 

  fprintf(fpOutputFile, "%s\n", 
	  gcppcSLIDEReservedWords[sliderwtEndFace]);
}

UINT_32 CSLIDEFace::GetPointCount()
{
  return m_listCSLIDEPoint.GetLength();
}

//////////////////////////////////////////////////////////////////////////////
// CSLIDEFaceIter Class
//

CSLIDEFaceIter::CSLIDEFaceIter()
{
}

CSLIDEFaceIter::~CSLIDEFaceIter()
{
}

VOID CSLIDEFaceIter::Init(CSLIDEFace *pCSLIDEFace)
{
  CSLIDEPrimitiveIter::Init(pCSLIDEFace);

  m_iter.Init(&(pCSLIDEFace->m_listCSLIDEPoint));
}

VOID CSLIDEFaceIter::Uninit()
{
  CSLIDEPrimitiveIter::Uninit();
}

CSLIDEPoint *CSLIDEFaceIter::PeekFirstCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekFirst());
}

CSLIDEPoint *CSLIDEFaceIter::PeekNextCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekNext());
}

CSLIDEPoint *CSLIDEFaceIter::PeekLastCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekLast());
}

CSLIDEPoint *CSLIDEFaceIter::PeekPrevCSLIDEPoint()
{
  return (CSLIDEPoint*)(m_iter.PeekPrev());
}
