#ifndef __SLIDEOBJECT_H__
#define __SLIDEOBJECT_H__

#include "slidenode.h"

class CSLIDEWorld;
class CSLIDESurface;
class CSLIDEFace;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEObject Class
//

class CSLIDEObjectIter;

class CSLIDEObject : public CSLIDENode
{
//////////////////////////////////////////////////
// Friends
//
public:
  friend class CSLIDEObjectIter;

//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEObject();
  virtual ~CSLIDEObject();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcID, 
		    CListInt *plistFaceIDs);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEPrimitiveIter *AllocateIter();
  virtual VOID FreeIter(CSLIDEPrimitiveIter *pCSLIDEObjectIter);

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual CSLIDECamera *FindCamera(CListIterInt *piterIDs, 
				   CListInt *plistCSLIDEInstance);
  virtual CSLIDELight *FindLight(CListIterInt *piterIDs, 
				 CListInt *plistCSLIDEInstance);

////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  virtual VOID Preprocess();
  virtual VOID PreprocessGraph();

  virtual VOID Update();
  virtual VOID UpdateGraph();
  virtual VOID InsertIntoBound(const Matrix &m, CBound &bound);

protected:
  virtual VOID CheckDynamic();
  virtual VOID UpdateState();
  virtual VOID CalculateBound();

//////////////////////////////////////////////////
// Rendering
//
public:
  // The Rendering Method is in the CSLIDENode class because it should 
  // be the same for both CSLIDEObjects and CSLIDEGroups

protected:
  virtual VOID ModifySurfacePointer(CSLIDESurface* &pCSLIDESurface);
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////
  virtual VOID ModifyShadingFlag(SLIDEShadingType &eShading);
  virtual VOID ModifyLODFlag(SLIDELODType &eLOD);

////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 3: These are the methods you will be dealing with
// 
  virtual VOID RenderGeometry(const Matrix &mGvp_proj, 
			      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
			      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
			      CBound *pCBoundClip, 
			      CListInt *plistLights,
			      CSLIDESurface *pCSLIDESurface,
			      SLIDEShadingType eShading, 
			      SLIDELODType eLOD);

public:
  virtual VOID RenderBound(const Matrix &mGvp_proj, const Matrix &mGproj_vrc, 
			   const Matrix &mGvrc_obj);
// 
// END  : ASSIGNMENT 3: These are the methods you will be dealing with
////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  virtual inline VOID SetSurfaceID(CHAR *pcSurfaceID);
  virtual inline VOID SetShading(DynInt *pdyniShading);
  virtual inline VOID SetLOD(DynInt *pdyniLOD);
  virtual inline VOID SetSolid(DynInt *pdyniSolid);

  virtual inline NodeType GetNodeType();

  virtual inline const CBound *GetCBound();
  virtual inline BOOL GetBoundEmpty();

  virtual UINT_32 GetFaceCount();

//////////////////////////////////////////////////
// Class Variables
//
protected:

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CHAR *m_pcSurfaceID;
  CSLIDESurface *m_pCSLIDESurface; // The surface which gives color
  DynInt m_dyniShading;            // Don't worry about the shading flag
  DynInt m_dyniLOD;                // Don't worry about the LOD flag
  DynInt m_dyniSolid;              // Don't worry about the solid flag
  CListInt *m_plistFaceIDs;
  CListInt m_listCSLIDEFace;       // List of CSLIDEFaces

  CBound m_bound;                  // Axis Aligned Bounding Box object
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID CSLIDEObject::SetSurfaceID(CHAR *pcSurfaceID)
{
  m_pcSurfaceID = pcSurfaceID;
}

inline VOID CSLIDEObject::SetShading(DynInt *pdyniShading)
{
  m_dyniShading.Set(pdyniShading);
}

inline VOID CSLIDEObject::SetLOD(DynInt *pdyniLOD)
{
  m_dyniLOD.Set(pdyniLOD);
}

inline VOID CSLIDEObject::SetSolid(DynInt *pdyniSolid)
{
  m_dyniSolid.Set(pdyniSolid);
}

inline CSLIDENode::NodeType CSLIDEObject::GetNodeType()
{
  return ntObject;
}

inline const CBound *CSLIDEObject::GetCBound()
{
  return &m_bound;
}

inline BOOL CSLIDEObject::GetBoundEmpty()
{
  return m_bound.GetEmpty();
}

//////////////////////////////////////////////////////////////////////////////
// CSLIDEObjectIter Class
//

class CSLIDEObjectIter : public CSLIDEPrimitiveIter
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEObjectIter();
  virtual ~CSLIDEObjectIter();
  virtual VOID Init(CSLIDEObject *pCSLIDEObject);
  virtual VOID Uninit();

//////////////////////////////////////////////////
// Iteration Functions
//
public:
  virtual CSLIDEFace *PeekFirstCSLIDEFace();
  virtual CSLIDEFace *PeekNextCSLIDEFace();

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CListIterInt m_iter;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//


#endif // __SLIDEOBJECT_H__
