#ifndef __SLIDEPARSER_H__
#define __SLIDEPARSER_H__

#include "parser.h"
#include "dynamics.h"
#include "point.h"
#include "vector.h"

#include "slide.h"
class CSLIDEWorld;
class CSLIDERender;
class CWindow;
class CSLIDEViewport;
class CSLIDELight;
class CSLIDELightPath;
class CSLIDECamera;
class CSLIDECameraPath;
class CSLIDENode;
class CSLIDEGroup;
class CSLIDEInstance;
class CSLIDETransform;
class CSLIDETranslate;
class CSLIDEScale;
class CSLIDERotate;
class CSLIDELookAt;
class CSLIDEObject;
class CSLIDEFace;
class CSLIDEPoint;
class CSLIDESurface;
//class Matrix;

//////////////////////////////////////////////////////////////////////////////
// CSLIDEParser Class
//

class CSLIDEParser : public CParser
{
//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDEParser();
  virtual ~CSLIDEParser();
  virtual VOID Init(CHAR *pcFile, CHAR **ppcDirectories, UINT_32 uiDirectories, 
		    CSLIDEParser **ppyy_CSLIDEParser, 
		    YYLTYPE *pyylloc, YY_PARSE_TYPE *pfyy_Parse,
		    CScanner **ppyy_CScanner,
		    int *pyyleng,
		    YY_BEGIN_INITIAL_TYPE *pfyy_BeginInitial,
		    YY_GET_CURRENT_BUFFER_TYPE *pfyy_GetCurrentBuffer,
		    YY_CREATE_BUFFER_TYPE *pfyy_CreateBuffer,
		    YY_DELETE_BUFFER_TYPE *pfyy_DeleteBuffer,
		    YY_SWITCH_TO_BUFFER_TYPE *pfyy_SwitchToBuffer,
		    YY_SET_VALUE_INT_TYPE *pfyy_SetValueInt,
		    YY_SET_VALUE_FLOAT_TYPE *pfyy_SetValueFloat,
		    YY_SET_VALUE_STRING_TYPE *pfyy_SetValueString);
  virtual VOID Uninit();

//////////////////////////////////////////////////
// AST Functions
//
public:
  virtual VOID AddRender(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			 CHAR *pcViewportID, CListInt *plistCameraPath, 
			 CHAR *pcRootID);
  virtual VOID EndRender();
  virtual VOID SetRenderTclTag(CHAR *pcTclTag);
  virtual VOID SetRenderLOD(DynInt *pdyniLOD);
  virtual VOID AddRenderLightPath(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
				  CListInt *plistLightPath, DynInt *pdyniOnOff);
  virtual VOID AddWindow(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			  CHAR *pcID);
  virtual VOID EndWindow();
  virtual VOID SetWindowOrigin(FLOAT fX, FLOAT fY);
  virtual VOID SetWindowSize(FLOAT fX, FLOAT fY);
  virtual VOID SetWindowBackground(DynVector *pdynv);
  virtual VOID AddViewport(CHAR *pcFile, UINT_32 uiLine, 
			   UINT_32 uiColumn,
			   CHAR *pcID, CHAR *pcWindowID);
  virtual VOID EndViewport();
  virtual VOID SetViewportOrigin(DynVector *pdynv);
  virtual VOID SetViewportSize(DynVector *pdynv);
  virtual VOID AddCamera(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			  CHAR *pcID);
  virtual VOID EndCamera();
  virtual VOID SetCameraProjection(DynInt *pdyniProjection);
  virtual VOID SetCameraFrustum(DynVector *pdynvMin, 
				DynVector *pdynvMax);
  virtual VOID AddLight(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			CHAR *pcID);
  virtual VOID EndLight();
  virtual VOID SetLightType(DynInt *pdyniType);
  virtual VOID SetLightColor(DynVector *pdynvColor);
  virtual VOID SetLightDeadDistance(DynFloat *pdynfDeadDistance);
  virtual VOID SetLightFallOff(DynFloat *pdynfFallOff);
  virtual VOID SetLightAngularFallOff(DynFloat *pdynfAngularFallOff);
  virtual VOID SetNodeSurfaceID(CHAR *pcSurfaceID);
  virtual VOID SetNodeShading(DynInt *pdyniShading);
  virtual VOID SetNodeLOD(DynInt *pdyniLOD);
  virtual VOID AddGroup(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			CHAR *pcID);
  virtual VOID EndGroup();
  virtual VOID AddInstance(CHAR *pcFile, UINT_32 uiLine, 
			   UINT_32 uiColumn, CHAR *pcChildID);
  virtual VOID EndInstance();
  virtual VOID SetInstanceID(CHAR *pcID);
  virtual VOID AddTranslate(CHAR *pcFile, UINT_32 uiLine, 
			    UINT_32 uiColumn,
			    DynVector *pdynv);
  virtual VOID AddScale(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			DynVector *pdynv);
  virtual VOID AddRotate(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			 DynVector *pdynvAxis, DynFloat *pdynfAngle);
  virtual VOID AddLookAt(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn);
  virtual VOID EndLookAt();
  virtual VOID SetLookAtEye(DynVector *pdynvEye);
  virtual VOID SetLookAtTarget(DynVector *pdynvTarget);
  virtual VOID SetLookAtUp(DynVector *pdynvUp);
/*
  virtual VOID AddGeneral(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			  FLOAT f0, FLOAT f1, FLOAT f2, FLOAT f3, 
			  FLOAT f4, FLOAT f5, FLOAT f6, FLOAT f7, 
			  FLOAT f8, FLOAT f9, FLOAT f10, FLOAT f11, 
			  FLOAT f12, FLOAT f13, FLOAT f14, FLOAT f15);
*/
  virtual VOID AddObject(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			 CHAR *pcID, 
			 CListInt *plistFaceIDs);
  virtual VOID EndObject();
  virtual VOID SetObjectSolid(DynInt *pdyniSolid);
  virtual VOID AddFace(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
		       CHAR *pcID, CListInt *plistPointIDs);
  virtual VOID EndFace();
  virtual VOID SetFaceSurfaceID(CHAR *pcSurfaceID);
  virtual VOID AddPoint(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			CHAR *pcID, DynPoint *pdynpt);
  virtual VOID EndPoint();
  virtual VOID SetPointSurfaceID(CHAR *pcSurfaceID);
  virtual VOID SetPointNormal(DynVector *pdynv);
  virtual VOID SetPointTexture(DynPoint *pdynpt);
  virtual VOID AddSurface(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn,
			  CHAR *pcID);
  virtual VOID EndSurface();
  virtual VOID SetSurfaceColor(DynVector *pdynv);
  virtual VOID SetSurfaceReflectivity(DynVector *pdynv);
  virtual VOID SetSurfaceExponent(DynFloat *pdynfExponent);
  virtual VOID SetSurfaceMetallic(DynFloat *pdynfMetallic);
  virtual VOID AddGlobalInitTclBlock(CHAR *pcTclBlock);
  virtual VOID AddGlobalUpdateTclBlock(CHAR *pcTclBlock);
  virtual CListInt *CreateIDList();
  virtual CListInt *InsertIDList(CListInt *pCListInt, CHAR *pcID);

protected:
  virtual CSLIDEWorld *CreateWorld();
  virtual CSLIDERender *CreateRender(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn,
				     CHAR *pcViewportID, 
				     CListInt *plistCameraPath, 
				     CHAR *pcRootID);
  virtual CSLIDELightPath *CreateLightPath(CHAR *pcFile, UINT_32 uiLine, 
					   UINT_32 uiColumn,
					   CListInt *plistLightPath,
					   DynInt *pdyniOnOff); 

// Cheap WIN32 hack
#ifdef CreateWindowA
#undef CreateWindowA
#endif


  virtual CWindow *CreateWindow(CHAR *pcFile, UINT_32 uiLine, 
				UINT_32 uiColumn,
				CHAR *pcID);
  virtual CSLIDEViewport *CreateViewport(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn,
					 CHAR *pcID, CHAR *pcWindowID);
  virtual CSLIDECamera *CreateCamera(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn,
				     CHAR *pcID);
  virtual CSLIDELight *CreateLight(CHAR *pcFile, UINT_32 uiLine, 
				   UINT_32 uiColumn,
				   CHAR *pcID);
  virtual CSLIDEGroup *CreateGroup(CHAR *pcFile, UINT_32 uiLine, 
				   UINT_32 uiColumn,
				   CHAR *pcID);
  virtual CSLIDEInstance *CreateInstance(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn, CHAR *pcChildID);
  virtual CSLIDETranslate *CreateTranslate(CHAR *pcFile, UINT_32 uiLine, 
					   UINT_32 uiColumn,
					   DynVector *pdynv);
  virtual CSLIDEScale *CreateScale(CHAR *pcFile, UINT_32 uiLine, 
				   UINT_32 uiColumn,
				   DynVector *pdynv);
  virtual CSLIDERotate *CreateRotate(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn,
				     DynVector *pdynvAxis,
				     DynFloat *pdynfAngle);
  virtual CSLIDELookAt *CreateLookAt(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn);
/*
  virtual CSLIDETransform *CreateGeneral(CHAR *pcFile, UINT_32 uiLine, 
					 UINT_32 uiColumn,
					 const Matrix &m);
*/
  virtual CSLIDEObject *CreateObject(CHAR *pcFile, UINT_32 uiLine, 
				     UINT_32 uiColumn,
				     CHAR *pcID, 
				     CListInt *plistFaceIDs);
  virtual CSLIDEFace *CreateFace(CHAR *pcFile, UINT_32 uiLine, 
				 UINT_32 uiColumn,
				 CHAR *pcID, 
				 CListInt *plistPointIDs);
  virtual CSLIDEPoint *CreatePoint(CHAR *pcFile, UINT_32 uiLine, 
				   UINT_32 uiColumn,
				   CHAR *pcID, 
				   DynPoint *pdynpt);
  virtual CSLIDESurface *CreateSurface(CHAR *pcFile, UINT_32 uiLine, 
				       UINT_32 uiColumn,
				       CHAR *pcID);
  CListInt *CreateList();
  CListInt *InsertList(CListInt *pCListInt, VOID *pvData);

//////////////////////////////////////////////////
// Accessor Functions
//
public:

//////////////////////////////////////////////////
// Class Variables
//

//////////////////////////////////////////////////
// Member Variables
//
protected:
  CSLIDEWorld *m_pCSLIDEWorld;
  CSLIDERender *m_pCSLIDERenderCurr;
  CWindow *m_pCWindowCurr;
  CSLIDEViewport *m_pCSLIDEViewportCurr;
  CSLIDECamera *m_pCSLIDECameraCurr;
  CSLIDELight *m_pCSLIDELightCurr;
  CSLIDENode *m_pCSLIDENodeCurr;
  CSLIDEGroup *m_pCSLIDEGroupCurr;
  CSLIDEInstance *m_pCSLIDEInstanceCurr;
  CSLIDEObject *m_pCSLIDEObjectCurr;
  CSLIDEFace *m_pCSLIDEFaceCurr;
  CSLIDEPoint *m_pCSLIDEPointCurr;
  CSLIDESurface *m_pCSLIDESurfaceCurr;
  CSLIDELookAt *m_pCSLIDELookAtCurr;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

//////////////////////////////////////////////////////////////////////
// Globals
//

extern CSLIDEParser *slide_InitParser(CSLIDEParser *pCSLIDEParser, 
				      CHAR *pcInput, 
				      CHAR **ppcDirectories, 
				      UINT_32 uiDirectories);

#endif // __SLIDEPARSER_H__
